from .main import RateLimit as RateLimit
from .redis import RedisInterface as RedisInterface, redisinterface as redisinterface

__all__ = ("RateLimit", "RedisInterface", "redisinterface")

__author__ = "PredaaA"
__version__ = "0.1.2"



from .redis import RedisInterface
from .utils import calc_ttl, parse_rate


class RateLimit:
    def __init__(self, redis: RedisInterface):
        self.redis = redis.pool
        self.encoding = redis._encoding

    async def _handling(self, rate: str, key: str, value: str) -> dict:
        count, seconds = parse_rate(rate)
        redis_key = f"[{key}][{value}][{count}/{seconds}]"

        current = await self.redis.get(redis_key, encoding=self.encoding)
        if current:
            current = int(current)
            if current >= count:
                return {
                    "ratelimited": True,
                    "data": {
                        "limit": count,
                        "remaining": count - (current if current else 0),
                        "reset": calc_ttl(await self.redis.ttl(redis_key)),
                    },
                }

        value = await self.redis.incr(redis_key)
        if value == 1:
            await self.redis.expire(redis_key, seconds)

        return {
            "ratelimited": False,
            "data": {
                "limit": count,
                "remaining": count - (current if current else 0),
                "reset": calc_ttl(await self.redis.ttl(redis_key)),
            },
        }

    async def set(self, rate: str, key: str, value: str) -> None:
        await self._handling(rate, key, value)

    async def get(self, rate: str, key: str, value: str) -> dict:
        return await self._handling(rate, key, value)



import aioredis
from typing import NoReturn
from .exceptions import RedisConnectionFail, RedisMissingConnection


class RedisPool:
    """Represent an Redis pool.

    Attributes
    ----------
    host: str = "localhost"
        The Redis host.
    port: int = 6379
        The Redis port.
    db: int = 0
        The Redis db.
    encoding: str = "utf-8"
        The encoding used.
    """

    __slots__ = ("_host", "_port", "_db", "_encoding")

    def __init__(
        self, host: str = "localhost", port: int = 6379, db: int = 0, encoding: str = "utf-8"
    ):
        self._host = host
        self._port = port
        self._db = db
        self._encoding = encoding

    def __repr__(self):
        return f"<RedisPool _host={self._host} _port={self._port} _db={self._db} _encoding={self._encoding}>"


class RedisInterface(RedisPool):

    __slots__ = "_redis"

    def __init__(self, **kwargs):
        self._redis = None
        super().__init__(**kwargs)

    async def init_connection(self) -> None:
        try:
            self._redis = await aioredis.create_redis_pool(
                address=f"redis://{self._host}:{self._port}",
                db=self._db,
                encoding=self._encoding,
                maxsize=50,
            )
        except Exception:
            raise RedisConnectionFail(
                "Something went wrong while trying to connect to this Redis DB."
            )

    async def teardown(self) -> None:
        if self._redis:
            self._redis.close()

    @property
    def pool(self) -> aioredis.Redis:
        if self._redis is None:
            raise RedisMissingConnection(
                "You need to have an valid Redis connection to use that property."
            )
        return self._redis

    @pool.setter
    async def pool(self, val) -> NoReturn:
        raise RuntimeError()

redisinterface = RedisInterface()



class RateLimiterException(Exception):
    """Base exception of ratelimiter-redis package."""


class RedisConnectionFail(RateLimiterException):
    """Raised when connecting to the Redis DB failed."""


class RedisMissingConnection(RateLimiterException):
    """Raise when ratelimiter-redis is not connected to Redis."""



import re
import time
from .exceptions import RateLimiterException


# The method above come from:
# https://github.com/romantomjak/redis-ratelimit/blob/master/redis_ratelimit/utils.py#L7-L26
def parse_rate(rate: str):
    rate_re = re.compile(r"([\d]+)/([\d]*)([smhd])")
    UNITS = {"s": 1, "m": 60, "h": 60 * 60, "d": 24 * 60 * 60}
    try:
        count, factor, unit = rate_re.match(rate).groups()
        count = int(count)
        seconds = UNITS[unit.lower()]
        if factor:
            seconds *= int(factor)
        return count, seconds
    except ValueError:
        raise RateLimiterException("Invalid rate value")


def calc_ttl(seconds: int):
    return int(time.time() + (seconds))
